/*
 * Copyright (c) 2019 Spotify AB.
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"): you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.spotify.heroic.suggest

import com.spotify.heroic.common.DateRange
import com.spotify.heroic.common.RequestTimer
import com.spotify.heroic.common.Series
import com.spotify.heroic.metric.RequestError
import eu.toolchain.async.Collector

data class WriteSuggest(
    val errors: List<RequestError> = listOf(),
    val times: List<Long> = listOf(),
    val ids: List<String> = listOf()
) {
    companion object {
        @JvmStatic fun timer(): RequestTimer<WriteSuggest> =
            RequestTimer { time: Long -> WriteSuggest(times = listOf(time)) }

        @JvmStatic fun reduce(): Collector<WriteSuggest, WriteSuggest> {
            return Collector { requests: Collection<WriteSuggest> ->
                val errors = requests.flatMap { it.errors }
                val times = requests.flatMap { it.times }
                WriteSuggest(errors = errors, times = times)
            }
        }
    }

    data class Request(val series: Series, val range: DateRange)
}
